xquery version "3.1";

module namespace macro-sort="http://fontane-nb.dariah.eu/macro-sort";

import module namespace config="http://textgrid.de/ns/SADE/config" at "../../config/config.xqm";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xi="http://www.w3.org/2001/XInclude";



declare function macro-sort:main($tei as node()*, $uri as xs:string) as node()* {
    let $header := $tei//tei:teiHeader
    let $sorted := macro-sort:sort($tei//tei:sourceDoc)
(:    let $cleared := macro-sort:clear($sorted):)
    let $cleared := $sorted

    let $final-tei :=
        element tei:TEI {
            $header,
            $cleared
        }

    let $store := xmldb:store($config:data-root || "/print/xml/", $uri || "-macrosort.xml", $final-tei)

    return
        $final-tei
 };

 declare function macro-sort:sort($nodes as node()*) as node()* {
    for $node in $nodes return
    typeswitch ($node)

    case text() return
        $node

    case comment() return
        $node

    case element(tei:milestone) return
        if($node/@unit = "section") then
            let $end-point-id := replace($node/@spanTo, "#", "")
            let $end-point := $node/root()//tei:anchor[@xml:id = $end-point-id]
            let $number := substring-after($node/@type, "Text_")
            let $handshift := $node/preceding::tei:milestone[@unit = "handshift"][1]

            return
                element tei:div {
                    attribute n {$number},
(:                    $node/(@* except @spanTo),:)
                    $node/@*,
                    $handshift,
                    (: all nodes on the current surface :)
                    $node/following-sibling::node()[. << $end-point],
                    $node/../following-sibling::node()[. << $end-point],
                    (: all complete surfaces before the anchor :)
                    $node/ancestor::tei:surface/following-sibling::tei:surface[. << $end-point],
                    (: all elements on the surface that contains the anchor which are before the anchor :)
                    $node/ancestor::tei:surface/following-sibling::tei:surface[descendant::tei:anchor[@xml:id = $end-point-id]]//tei:anchor[@xml:id = $end-point-id]/preceding-sibling::*
                }

        else
            macro-sort:copy-element($node)

    default return
        let $section-marker := $node/preceding::tei:milestone[@unit = "section"][1]
        let $end-point-id := replace($section-marker/@spanTo, "#", "")
        let $end-point := $node/root()//tei:anchor[@xml:id = $end-point-id]

        return
            if($node[. >> $section-marker and . << $end-point]) then
                ()
            else
                macro-sort:copy-element($node)
 };


declare function macro-sort:copy-element($node as element(*)) as element(*) {
    element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
        $node/@*,
        macro-sort:sort($node/node())
    }
};

declare function macro-sort:clear($nodes as node()*) as node()* {
    for $node in $nodes return
    typeswitch ($node)

    case text() return
        $node

    case comment() return
        $node

    default return
        if($node/@outside-section = "true") then
            ()
        else
            $node
};
